/*
 * irreco - Ir Remote Control
 * Copyright (C) 2007,2008  Arto Karppinen (arto.karppinen@iki.fi)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "irreco_webdb_client.h"

#define IRRECO_WEBDB_URL "http://mercury.wipsl.com/irreco/webdb/"
/*#define IRRECO_WEBDB_URL "http://193.65.64.89/irreco/webdb/"*/
/*
for php files
svn co svn+irreco:///irreco/irreco/database/trunk/php /var/www/irreco/webdb
*/


/** 
 * @typedef IrrecoWebdbClient
 * 
 * Irreco Web Database API implementation. These functions connect to the 
 * IrrecoWebdbClient trough XML-RPC connection, pass arguments, and decode return
 * values.
 */



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Prototypes                                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

static gboolean irreco_webdb_client_check_error();
static void irreco_webdb_client_reset_env();



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Construction & Destruction                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * Create new IrrecoWebdbClient Object
 */
IrrecoWebdbClient *irreco_webdb_client_new()
{
	IrrecoWebdbClient *self;
	IRRECO_ENTER
	
	self = g_slice_new0(IrrecoWebdbClient);
	self->error_env = g_slice_new0(xmlrpc_env);
	xmlrpc_env_init(self->error_env);
	self->server = xmlrpc_server_info_new(self->error_env,
					      IRRECO_WEBDB_URL);
	if (irreco_webdb_client_check_error(self)) {
		self->server = NULL;
		irreco_webdb_client_free(self);
		IRRECO_RETURN_PTR(NULL);
	}
	IRRECO_RETURN_PTR(self);
}

void irreco_webdb_client_free(IrrecoWebdbClient *self)
{
	IRRECO_ENTER
	
	g_assert(self != NULL);
	
	if (self->server) xmlrpc_server_info_free(self->server);
	xmlrpc_env_clean(self->error_env);
	
	g_slice_free(xmlrpc_env, self->error_env);
	self->error_env = NULL;
	
	g_slice_free(IrrecoWebdbClient, self);
	self = NULL;
	
	IRRECO_RETURN
}



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Private Functions                                                          */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * Release of RPC results and errors.
 */
static void irreco_webdb_client_reset_env(IrrecoWebdbClient *self)
{
	IRRECO_ENTER
	if (irreco_webdb_client_check_error(self)) {
		xmlrpc_env_clean(self->error_env);
		xmlrpc_env_init(self->error_env);
	}
	IRRECO_RETURN
}

/**
 * Check if the previous call to XMLRPC-C succeeded.
 *
 * @return TRUE if error occured, FALSE otherwise.
 */
static gboolean irreco_webdb_client_check_error(IrrecoWebdbClient *self)
{
	IRRECO_ENTER
	if(self->error_env->fault_occurred) {
		IRRECO_ERROR("%d: %s\n", self->error_env->fault_code,
			     self->error_env->fault_string);
		IRRECO_RETURN_BOOL(TRUE);
	}
	IRRECO_RETURN_BOOL(FALSE);
}



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Functions                                                                  */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * Get error message.
 *
 * @param msg GString object to recieve the error message.
 */
void irreco_webdb_client_get_error_msg(IrrecoWebdbClient *self, GString *msg)
{
	IRRECO_ENTER
	g_string_printf(msg, "%d: %s\n", 
			self->error_env->fault_code,
			self->error_env->fault_string);
	IRRECO_RETURN
}

/**
 * Ask WebDB to sum two numbers.
 *
 * @param num_a	First number.
 * @param num_b Second number.
 * @param sum	Variable to recieve the result.
 * @return TRUE on successfull XML-RPC call, FALSE otherwise.
 */
gboolean irreco_webdb_client_sum(IrrecoWebdbClient *self, 
				 gint num_a, 
				 gint num_b, 
				 gint *sum)
{
	gboolean	 rvalue		= FALSE;
	xmlrpc_value	*response	= NULL;
	IRRECO_ENTER
	
	irreco_webdb_client_reset_env(self);
	response = xmlrpc_client_call_server(self->error_env, self->server, 
					     "sum", "(ii)", 
					     (xmlrpc_int32) num_a, 
					     (xmlrpc_int32) num_b);
	if (irreco_webdb_client_check_error(self)) {
		response = NULL;
		goto cleanup;
	}
	
	xmlrpc_read_int(self->error_env, response, sum);
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	
	/* No error occured. */
	rvalue = TRUE;
	
	cleanup:
	if (response) xmlrpc_DECREF(response);
	IRRECO_RETURN_BOOL(rvalue);
}

/**
 * 
 */

gboolean irreco_webdb_client_add_user(IrrecoWebdbClient *self, 
				      const gchar *name, 
				      const gchar *email,
				      const gchar *passwd)
{
	gboolean	rvalue			= FALSE;
	xmlrpc_value	*response		= NULL;
	
	IRRECO_ENTER
	
	irreco_webdb_client_reset_env(self);
	
	/* Execute XML-RPC call. */
	response = xmlrpc_client_call_server(self->error_env, 
				     self->server, 
				     "addUser", 
				     "(sss)", 
				     name,
				     email,
				     passwd);
	
	if (irreco_webdb_client_check_error(self)) {
		response = NULL;
		if (response) xmlrpc_DECREF(response);
		IRRECO_RETURN_BOOL(FALSE);
	}
	
	/* Parse response and set to user_exists */
	xmlrpc_read_bool(self->error_env, response, &rvalue);
	
	if (irreco_webdb_client_check_error(self)) {
		response = NULL;
		if (response) xmlrpc_DECREF(response);
		IRRECO_RETURN_BOOL(FALSE);
	}
	
	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_upload_configuration(IrrecoWebdbClient *self,
						  const gchar *backend,
						  const gchar *category,
						  const gchar *file_hash,
						  const gchar *file_name,
						  const gchar *manufacturer,
						  const gchar *model,
						  const gchar *password,
						  const gchar *user,
						  const gchar *data)
{
	gboolean	rvalue		= FALSE;
	xmlrpc_value	*response	= NULL;
	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);
	
	/* Execute XML-RPC call. */
	response = xmlrpc_client_call_server(self->error_env, 
				     self->server, 
				     /* For testing purposes only */
				     "uploadConfiguration", 
				     "(sssssssss)", 
				     backend,
				     category,
	 			     manufacturer,
				     model,
				     user,
				     password,
	 			     file_hash,
				     file_name,
				     data);
	
	if (irreco_webdb_client_check_error(self)) {
		response = NULL;
		goto cleanup;
	}

	/* No error occured. */
	rvalue = TRUE;
	
	cleanup:
	if (response) xmlrpc_DECREF(response);

	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_categories(IrrecoWebdbClient *self, 
					    IrrecoStringTable **category_list)
{
	gint		i		= 0;
	gint		array_size	= 0;
	gboolean	rvalue		= FALSE;
	xmlrpc_value	*response	= NULL;
	xmlrpc_value	*category_xml	= NULL;
	const gchar 	*category_str	= NULL;
	IRRECO_ENTER
	
	*category_list = NULL;
	irreco_webdb_client_reset_env(self);
	
	/* Execute XML-RPC call. */
	response = xmlrpc_client_call_server(self->error_env, self->server, 
					     "getCategories", "()");
	if (irreco_webdb_client_check_error(self)) {
		response = NULL;
		goto cleanup;
	}
	
	/* Get size responce of array. */
	array_size = xmlrpc_array_size(self->error_env, response);
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	
	/* Parse array. */
	*category_list = irreco_string_table_new(
		(GDestroyNotify) irreco_string_table_free, NULL);
	for(i = 0; i < array_size; i++) {
		xmlrpc_array_read_item(self->error_env, response, 
				       i, &category_xml);
		if (irreco_webdb_client_check_error(self)) goto cleanup;
		
		xmlrpc_read_string(self->error_env, category_xml, 
				   &category_str);
		if (irreco_webdb_client_check_error(self)) goto cleanup;
		
		IRRECO_DEBUG("Category: %s\n", category_str);
		irreco_string_table_add(*category_list, category_str, NULL);
		xmlrpc_DECREF(category_xml);
	}
	
	/* No error occured. */
	rvalue = TRUE;
	
	cleanup:
	if (response) xmlrpc_DECREF(response);
	if (category_xml) xmlrpc_DECREF(category_xml);
	if (rvalue == FALSE && *category_list != NULL) {
		irreco_string_table_free(*category_list);
	}
	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_all_categories(IrrecoWebdbClient *self, 
					    IrrecoStringTable **category_list)
{
	gint		i		= 0;
	gint		array_size	= 0;
	gboolean	rvalue		= FALSE;
	xmlrpc_value	*response	= NULL;
	xmlrpc_value	*category_xml	= NULL;
	const gchar 	*category_str	= NULL;
	IRRECO_ENTER
	
	*category_list = NULL;
	irreco_webdb_client_reset_env(self);
	
	/* Execute XML-RPC call. */
	response = xmlrpc_client_call_server(self->error_env, self->server, 
					     "getAllCategories", "()");
	if (irreco_webdb_client_check_error(self)) {
		response = NULL;
		goto cleanup;
	}
	
	/* Get size responce of array. */
	array_size = xmlrpc_array_size(self->error_env, response);
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	
	/* Parse array. */
	*category_list = irreco_string_table_new(
		(GDestroyNotify) irreco_string_table_free, NULL);
	for(i = 0; i < array_size; i++) {
		xmlrpc_array_read_item(self->error_env, response, 
				       i, &category_xml);
		if (irreco_webdb_client_check_error(self)) goto cleanup;
		
		xmlrpc_read_string(self->error_env, category_xml, 
				   &category_str);
		if (irreco_webdb_client_check_error(self)) goto cleanup;
		
		IRRECO_DEBUG("Category: %s\n", category_str);
		irreco_string_table_add(*category_list, category_str, NULL);
		xmlrpc_DECREF(category_xml);
	}
	
	/* No error occured. */
	rvalue = TRUE;
	
	cleanup:
	if (response) xmlrpc_DECREF(response);
	if (category_xml) xmlrpc_DECREF(category_xml);
	if (rvalue == FALSE && *category_list != NULL) {
		irreco_string_table_free(*category_list);
	}
	IRRECO_RETURN_BOOL(rvalue);
}
gboolean irreco_webdb_client_get_manufacturers(IrrecoWebdbClient *self,
					const gchar *category, 
					IrrecoStringTable **manufacturer_list)
{
	gint		i			= 0;
	gint		array_size		= 0;
	gboolean	rvalue			= FALSE;
	xmlrpc_value	*response		= NULL;
	xmlrpc_value	*manufacturer_xml	= NULL;
	const gchar 	*manufacturer_str	= NULL;
	IRRECO_ENTER
	
	*manufacturer_list = NULL;
	irreco_webdb_client_reset_env(self);
	
	/* Execute XML-RPC call. */
	response = xmlrpc_client_call_server(self->error_env, self->server, 
					     "getManufacturers", "(s)",
	  				     category);
	if (irreco_webdb_client_check_error(self)) {
		response = NULL;
		goto cleanup;
	}
	
	/* Get size responce of array. */
	array_size = xmlrpc_array_size(self->error_env, response);
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	
	/* Parse array. */
	*manufacturer_list = irreco_string_table_new(
		(GDestroyNotify) irreco_string_table_free, NULL);
	for(i = 0; i < array_size; i++) {
		xmlrpc_array_read_item(self->error_env, response, 
				       i, &manufacturer_xml);
		if (irreco_webdb_client_check_error(self)) goto cleanup;
		
		xmlrpc_read_string(self->error_env, manufacturer_xml, 
				   &manufacturer_str);
		if (irreco_webdb_client_check_error(self)) goto cleanup;
		
		IRRECO_DEBUG("Manufacturer: %s\n", manufacturer_str);
		irreco_string_table_add(*manufacturer_list,
					 manufacturer_str, NULL);
		xmlrpc_DECREF(manufacturer_xml);
	}
	
	/* No error occured. */
	rvalue = TRUE;
	
	cleanup:
	if (response) xmlrpc_DECREF(response);
	if (manufacturer_xml) xmlrpc_DECREF(manufacturer_xml);
	if (rvalue == FALSE && *manufacturer_list != NULL) {
		irreco_string_table_free(*manufacturer_list);
	}
	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_all_manufacturers(IrrecoWebdbClient *self, 
			    IrrecoStringTable **manufacturer_list)
{
	gint		i			= 0;
	gint		array_size		= 0;
	gboolean	rvalue			= FALSE;
	xmlrpc_value	*response		= NULL;
	xmlrpc_value	*manufacturer_xml	= NULL;
	const gchar 	*manufacturer_str	= NULL;
	IRRECO_ENTER
	
	*manufacturer_list = NULL;
	irreco_webdb_client_reset_env(self);
	
	/* Execute XML-RPC call. */
	response = xmlrpc_client_call_server(self->error_env, self->server, 
					     "getAllManufacturers", "()");
	if (irreco_webdb_client_check_error(self)) {
		response = NULL;
		goto cleanup;
	}
	
	/* Get size response of array. */
	array_size = xmlrpc_array_size(self->error_env, response);
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	
	/* Parse array. */
	*manufacturer_list = irreco_string_table_new(
		(GDestroyNotify) irreco_string_table_free, NULL);
	for(i = 0; i < array_size; i++) {
		xmlrpc_array_read_item(self->error_env, response, 
				       i, &manufacturer_xml);
		if (irreco_webdb_client_check_error(self)) goto cleanup;
		
		xmlrpc_read_string(self->error_env, manufacturer_xml, 
				   &manufacturer_str);
		if (irreco_webdb_client_check_error(self)) goto cleanup;
		
		IRRECO_DEBUG("Manufacturer: %s\n", manufacturer_str);
		irreco_string_table_add(*manufacturer_list, 
				manufacturer_str, NULL);
		xmlrpc_DECREF(manufacturer_xml);
	}
	
	/* No error occured. */
	rvalue = TRUE;
	
	cleanup:
	if (response) xmlrpc_DECREF(response);
	if (manufacturer_xml) xmlrpc_DECREF(manufacturer_xml);
	if (rvalue == FALSE && *manufacturer_list != NULL) {
		irreco_string_table_free(*manufacturer_list);
	}
	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_models(IrrecoWebdbClient *self,
					const gchar *category,
					const gchar *manufacturer, 
					IrrecoStringTable **model_list)
{
	gint		i			= 0;
	gint		array_size		= 0;
	gboolean	rvalue			= FALSE;
	xmlrpc_value	*response		= NULL;
	xmlrpc_value	*model_xml		= NULL;
	const gchar 	*model_str		= NULL;
	IRRECO_ENTER
	
	*model_list = NULL;
	irreco_webdb_client_reset_env(self);
	
	/* Execute XML-RPC call. */
	response = xmlrpc_client_call_server(self->error_env, self->server, 
					     "getModels", "(ss)",
	  				     category,manufacturer);
	if (irreco_webdb_client_check_error(self)) {
		response = NULL;
		goto cleanup;
	}
	
	/* Get size responce of array. */
	array_size = xmlrpc_array_size(self->error_env, response);
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	
	/* Parse array. */
	*model_list = irreco_string_table_new(
		(GDestroyNotify) irreco_string_table_free, NULL);
	for(i = 0; i < array_size; i++) {
		xmlrpc_array_read_item(self->error_env, response, 
				       i, &model_xml);
		if (irreco_webdb_client_check_error(self)) goto cleanup;
		
		xmlrpc_read_string(self->error_env, model_xml, 
				   &model_str);
		if (irreco_webdb_client_check_error(self)) goto cleanup;
		
		IRRECO_DEBUG("Model: %s\n", model_str);
		irreco_string_table_add(*model_list,
					 model_str, NULL);
		xmlrpc_DECREF(model_xml);
	}
	
	/* No error occured. */
	rvalue = TRUE;
	
	cleanup:
	if (response) xmlrpc_DECREF(response);
	if (model_xml) xmlrpc_DECREF(model_xml);
	if (rvalue == FALSE && *model_list != NULL) {
		irreco_string_table_free(*model_list);
	}
	IRRECO_RETURN_BOOL(rvalue);
}


gboolean irreco_webdb_client_get_configs(IrrecoWebdbClient *self,
					 const gchar *category,
					 const gchar *manufacturer,
					 const gchar *model, 
					 IrrecoStringTable **config_list)
{
	gint		i			= 0;
	gint		array_size		= 0;
	gboolean	rvalue			= FALSE;
	xmlrpc_value	*response		= NULL;
	xmlrpc_value	*config_xml		= NULL;
	const gchar 	*config_str		= NULL;
	IRRECO_ENTER
	
	*config_list = NULL;
	irreco_webdb_client_reset_env(self);

	/* Execute XML-RPC call. */
	response = xmlrpc_client_call_server(self->error_env, self->server, 
					     "getConfigs", "(s)",
	  				     model);
	if (irreco_webdb_client_check_error(self)) {
		response = NULL;
		goto cleanup;
	}
	
	/* Get size responce of array. */
	array_size = xmlrpc_array_size(self->error_env, response);
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	
	/* Parse array. */
	*config_list = irreco_string_table_new(NULL, NULL);
	for(i = 0; i < array_size; i++) {

		xmlrpc_array_read_item(self->error_env, response, 
				       i, &config_xml);
		if (irreco_webdb_client_check_error(self)) goto cleanup;
		
		xmlrpc_read_string(self->error_env, config_xml, 
				   &config_str);
		
		if (irreco_webdb_client_check_error(self)) goto cleanup;
		
		IRRECO_DEBUG("Config: %s\n", config_str);
		irreco_string_table_add(*config_list, config_str, NULL);
		xmlrpc_DECREF(config_xml);
	}
	
	/* No error occured. */
	rvalue = TRUE;
	
	cleanup:
	if (response) xmlrpc_DECREF(response);
	if (config_xml) xmlrpc_DECREF(config_xml);
	if (rvalue == FALSE && *config_list != NULL) {
		irreco_string_table_free(*config_list);
	}
	IRRECO_RETURN_BOOL(rvalue);
}


gboolean irreco_webdb_client_get_configuration(IrrecoWebdbClient *self,
					       const gchar *id,
					       IrrecoWebdbConf **configuration)
{
	gboolean	rvalue		= FALSE;
	xmlrpc_value	*response	= NULL;	
	xmlrpc_value	*_xml		= NULL;
	const gchar	*user;
	const gchar	*backend;
	const gchar	*category;
	const gchar	*manufacturer;
	const gchar	*model;
	const gchar	*file_hash;
	const gchar	*file_name;
	const gchar	*uploaded;
	const gchar	*download_count;

	IRRECO_ENTER
			
	*configuration = irreco_webdb_conf_new();
	
	irreco_webdb_client_reset_env(self);

	/* Execute XML-RPC call. */
	response = xmlrpc_client_call_server(self->error_env, self->server, 
					     "getConfiguration", "(s)", id);
	if (irreco_webdb_client_check_error(self)) {
		response = NULL;
		goto cleanup;
	}
	
	/* Get size responce of array. */
	
	xmlrpc_struct_find_value(self->error_env, response, "user", &_xml);
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	xmlrpc_read_string(self->error_env, _xml, &user);
	xmlrpc_DECREF(_xml);
	
	xmlrpc_struct_find_value(self->error_env, response, "backend", &_xml);
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	xmlrpc_read_string(self->error_env, _xml, &backend);
	xmlrpc_DECREF(_xml);

	xmlrpc_struct_find_value(self->error_env, response, "category", &_xml);
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	xmlrpc_read_string(self->error_env, _xml, &category);
	xmlrpc_DECREF(_xml);
	
	xmlrpc_struct_find_value(self->error_env, response, "manufacturer",
				 &_xml);
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	xmlrpc_read_string(self->error_env, _xml, &manufacturer);
	xmlrpc_DECREF(_xml);
	
	xmlrpc_struct_find_value(self->error_env, response, "model", &_xml);
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	xmlrpc_read_string(self->error_env, _xml, &model);
	xmlrpc_DECREF(_xml);
	
	xmlrpc_struct_find_value(self->error_env, response, "file_hash", &_xml);
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	xmlrpc_read_string(self->error_env, _xml, &file_hash);
	xmlrpc_DECREF(_xml);
	
	xmlrpc_struct_find_value(self->error_env, response, "file_name", &_xml);
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	xmlrpc_read_string(self->error_env, _xml, &file_name);
	xmlrpc_DECREF(_xml);
	
	xmlrpc_struct_find_value(self->error_env, response, "uploaded", &_xml);
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	xmlrpc_read_string(self->error_env, _xml, &uploaded);
	xmlrpc_DECREF(_xml);
	
	xmlrpc_struct_find_value(self->error_env, response,
				 "download_count", &_xml);
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	xmlrpc_read_string(self->error_env, _xml, &download_count);
	xmlrpc_DECREF(_xml);
	
	if (irreco_webdb_client_check_error(self)) goto cleanup;

	IRRECO_DEBUG("Configuration: %d %s %s %s %s %s %s %s %s %s\n",
		     (gint)id, user, backend, category, manufacturer,
       		     model, file_hash, file_name, uploaded, download_count);
	
	irreco_webdb_conf_set(*configuration, (gint)id, user, backend, category,
			      manufacturer, model, file_hash, file_name,
			      uploaded, download_count);
	
	/* No error occured. */
	rvalue = TRUE;
	
	cleanup:
	if (response) xmlrpc_DECREF(response);
	if (_xml) xmlrpc_DECREF(_xml);

	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_file(IrrecoWebdbClient *self,
				      const gchar *file_hash,
				      const gchar *file_name,
				      GString **file_data)
{
	gboolean	rvalue		= FALSE;
	xmlrpc_value	*response	= NULL;	
	xmlrpc_value	*_xml		= NULL;
	const gchar	*data		= NULL;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);
	
	IRRECO_LINE

	/* Execute XML-RPC call. */
	response = xmlrpc_client_call_server(self->error_env, self->server, 
					     "getFile", "(ss)",
	  				     file_hash, file_name);
	if (irreco_webdb_client_check_error(self)) {
		response = NULL;
		goto cleanup;
	}
	
	/* Get size responce of array. */
	
	xmlrpc_struct_find_value(self->error_env, response, "data", &_xml);
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	xmlrpc_read_string(self->error_env, _xml, &data);
	xmlrpc_DECREF(_xml);
	
	if (irreco_webdb_client_check_error(self)) goto cleanup;

	IRRECO_DEBUG("File: %s\n", data);
	
	*file_data = g_string_new (data);
	
	/* No error occured. */
	rvalue = TRUE;
	
	cleanup:
	if (response) xmlrpc_DECREF(response);
	if (_xml) xmlrpc_DECREF(_xml);

	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_user_exists(IrrecoWebdbClient *self, 
					     const gchar *name, 
	  				     gboolean *user_exists)
{
	gboolean	 rvalue			= FALSE;
	xmlrpc_value	*response		= NULL;

	IRRECO_ENTER
	
	irreco_webdb_client_reset_env(self);
	
	/* Execute XML-RPC call. */	
	response = xmlrpc_client_call_server(self->error_env, 
					     self->server, 
					     "getUserExists", 
					     "(s)", 
					     name);
	
	if (irreco_webdb_client_check_error(self)) {
		response = NULL;
		goto cleanup;
	}
	
	/* Parse response and set to user_exists */
	xmlrpc_read_bool(self->error_env, response, user_exists);
	
	if (irreco_webdb_client_check_error(self)) goto cleanup;
	
	rvalue = TRUE;
	
	cleanup:
	if (response) xmlrpc_DECREF(response);
	
	IRRECO_RETURN_BOOL(rvalue);
}


#if 0
/**
 * 
 */
gboolean irreco_webdb_client_get_category_list(GHashTable **category_list)
{
	gint i;
	IRRECO_ENTER
	
	*category_list = NULL;
	
	/* Execute XML-RPC call. */
	irreco_webdb_client_reset_env();
	self->result = xmlrpc_client_call_server(self->error_env, self->server, 
						 "getCategoryList", "()");
	if (!irreco_webdb_client_is_ok()) IRRECO_RETURN_BOOL(FALSE);
	
	/* Get structure size. */
	i = xmlrpc_struct_size(self->error_env, self->result);
	if (!irreco_webdb_client_is_ok()) IRRECO_RETURN_BOOL(FALSE);
	
	/* Create hashtable. */
	*category_list = g_hash_table_new_full(g_int_hash, g_int_equal, 
					  NULL, g_free);
	
	/* Parse structure. */
	while (i--) {
		xmlrpc_value *xml_key;
		xmlrpc_value *xml_value;
		const gchar * str_key;
		const gchar * str_value;
		
		xmlrpc_struct_read_member(self->error_env, self->result, i,
					  &xml_key, &xml_value);
		if (!irreco_webdb_client_is_ok()) goto error;
		
		xmlrpc_read_string(self->error_env, xml_key, &str_key);
		if (!irreco_webdb_client_is_ok()) goto error;
		
		xmlrpc_read_string(self->error_env, xml_value, &str_value);
		if (!irreco_webdb_client_is_ok()) goto error;
		
		IRRECO_PRINTF("Key: %s, Val: %s\n", str_key, str_value);
		/*g_hash_table_add();*/

		xmlrpc_DECREF(xml_key);
		xmlrpc_DECREF(xml_value);
	}
	
	IRRECO_RETURN_BOOL(TRUE);
	
	error:
	g_hash_table_destroy(*category_list);
	*category_list = NULL;
	IRRECO_RETURN_BOOL(FALSE);
}
#endif


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Events and Callbacks                                                       */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/









